/* posacq1.c (was SERACQ1.c) serial acquisition
 * acquires serial data from a POSS and writes it to disk files
 * compile with  <gcc posacq1.c -o posacq1>
 * 2015-Jun-17 - PR: remove sscanf warning by removing pointer deref
                     moved serial_port settings into par file
                     init_serial_port now its own subroutine
                     always mkdir DATA_PATH
                     removed DATA_BASE
                     lengthened wait for serial message to 1-sec
 * 2013-Jun-12 - modified by VS and PR: comment to only accept POSS data --> ### POSACQ1_2013-Jun-12.c ###
 * 2013-Jun-13 - PR: excise rest of unused code and tidy up hard-coded variables --> ### POSACQ1_2013-Jun-13.c ###
 *                   removed summaryemail
 *                   comment output file's: "in case we are root, change the ownership to user_account"
 *                   removed half the unnecessary #includes
 */

#include <sys/stat.h> //for mkdir
#include <sys/timeb.h> //for struct timeb
#include <stdio.h> //for printf, fprintf
#include <time.h>  //for ctime()
#include <stdlib.h> //for exit()
#include <string.h> // strncpy, strcat
#include <term.h> //for struct termios
#include <fcntl.h> //for O_RDONLY, O_NOCTTY, O_NDELAY

#define _POSIX_SOURCE 1

#define VERDATE "POSACQ1 (build 2015-Jun-19)"  /* change this each edit */

#define DATA_PATH "./data/"

//#define SERIAL_BAUDRATE B9600
//#define SERIAL_BIT CS7
//#define SERIAL_PARITY PARENB //7 bit, even parity, 1 stop (default)
//#define SERIAL_STOP ~CSTOPB 

#define PARAMETER_FILE "posacq1.par"
#define MAIL_MSG_FILE "lastmailmsg"
#define POSS_DAT_FILE "POSyyyymmdd.txt"
#define POSS_MSG_FILE "MESPOSyyyymmdd.txt"

#define STRING_MAX_LENGTH 256

/* general variables */
int i; // loop var, can/will be compared with other signed vars
int oldmin,oldhr,olddayt,olddayu;
unsigned int nloops=0;  /* how many times through main loop in 1 min */
char syscmd[STRING_MAX_LENGTH];

/* POSS variables */
int fhR2;
int nchar2;
int pchar2=0;
char sposs[8192]={0};    /* one packet of data */
int posfm=0;      /* par fails count in minute */
int posfd=0;      /* par fails count in day */
int minnopos=0;      /* mins with no valid poss data */
char posfaultcode[5];    /* poss fault code string */
int posfaultnum=0;    /* poss fault integer (hex) number */
char posfault[STRING_MAX_LENGTH];    /* poss fault text string */

/* time variables */
struct timeb tstart;  /* ftime - start */
struct timeb tfinish;
struct timeb telaps;
struct timeb longtime;
struct tm *newtime;

char *stimline;
char *ftimline;
char *acqtimline;
unsigned char month;
char scurdatetim[21]="dd/mm/yyyy,hh:mm:ss,";  /* current date time string */

/* file variables */
FILE *fp,*fpp;
char full_param_file[STRING_MAX_LENGTH];
char full_mail_file[STRING_MAX_LENGTH];
char pos_basefile[STRING_MAX_LENGTH];  
char mespos_basefile[STRING_MAX_LENGTH];
char poslog[STRING_MAX_LENGTH];
char mesposlog[STRING_MAX_LENGTH];

/* tty port variables */
struct termios newtio;

/* keyboard variables */
static struct termios initial_settings, new_settings; 
char kbchar;
int kbptr=0;
char kbword[STRING_MAX_LENGTH]={0};
int posd=1;    /* dump good poss packets */
int posrawd=0;    /* dump all poss raw characters */

/* email variables */
int posmailsent=0;  /* pos mail sent this hour */

/* default parameter file variables */
char     pardate[STRING_MAX_LENGTH]="2015-Jun-17_PR";
char        site[STRING_MAX_LENGTH]="WXX";  /* 3 letter site id */
char    maildest[STRING_MAX_LENGTH]="peter.rodriguez@ec.gc.ca";  /* separate with ; */
char possendmail='Y';  /* Y to send email messages */

//7 bit, even parity, 1 stop (default)
char SERIAL_PORT[24]="/dev/ttyUSB0";
int  SERIAL_BAUDRATE=B9600;
int  SERIAL_BIT     =CS7;
int  SERIAL_PARITY  =PARENB;
int  SERIAL_STOP    =~CSTOPB;

/*******************************************************************/
main()
/*******************************************************************/
{
  curdatetim();        /* get current date time */
  startup();        /* init keyboard and create files */
  printf("\nOK, POSACQ1 all initialized! Awaiting data and/or keyboard input (i.e. type 'help')\n");

  while(1)  /* forever loop */
  {
    curdatetim();  /* get current date time */
    testday();  /* see if the day has changed */
    testhour();  /* see if the hour has changed */
    testmin();  /* see if the min has changed */

    do checkposs();    /* see if poss has new data */
    while(nchar2>0);

    checkkybd();  /* see if keyboard has a new command */
    nloops++;  /* count times through loop in 1 minute */  
//    usleep(125000);  /* sleep for 125 msec, tuned for nloops=~480/minute */
    sleep(1);  /* sleep for 1 sec, tuned for nloops=1/minute */
  }/* end of foerever loop */  
}/* end of main  */

/*******************************************************************/
startup()
/*******************************************************************/
{
//PR  ftime(&tstart);    /* for elapsed time */

  system("setterm -blank 0");  /* disable screen blanking */

  printf("Program-Version: %s\n",VERDATE); 
  
  stimline=ctime(&(tstart.time));
  acqtimline=ctime(&(tstart.time));
  if(acqtimline[8]==' ') acqtimline[8]='0';  /* gcc ctime uses space */
  if(acqtimline[15]=='1'||acqtimline[15]=='3'||acqtimline[15]=='5'||
    acqtimline[15]=='7'||acqtimline[15]=='9')
    acqtimline[15] = (acqtimline[15]-1);
  printf("Startup time = %.19s %s\n",stimline, &stimline[20]);

  /* initialize keyboard */
  tcgetattr(0,&initial_settings);
  new_settings=initial_settings;
  new_settings.c_lflag &= ~ICANON;
  new_settings.c_lflag &= ~ISIG;
  new_settings.c_cc[VMIN]  = 0;
  new_settings.c_cc[VTIME] = 0;
  tcsetattr(0, TCSANOW, &new_settings);
  
  /* initialize keyboard variables */
  kbptr=0;    /* for keyboard input */
  /* clear out kbword */
  for(i=0;i<80;i++) kbword[i]=0;
  printf("Keyboard initialization successful.\n");

  /* initialize the oldmin variable to the current minute */
  ftime(&longtime);    /* gets long time */
  acqtimline=ctime(&longtime.time);
  if(acqtimline[8]==' ') acqtimline[8]='0';  /* gcc ctime uses space */
  oldmin=acqtimline[15];

  mkdir(DATA_PATH,0775); /* make DATA_PATH directory */

  mkmesnm(); /* make today's filename for message log */
  mkposnm(); /* make today's filename for poss data */

  /* write to message file */
  /* this is normal, so don't increment the posfm counter */
  fp=fopen(mesposlog,"a");
  fprintf(fp,"%i,%s starting %s\n",posfd,scurdatetim,VERDATE);
  fclose(fp);

  readparfile();      /* get parameters */

  init_serial_port(); /* init tty port */

//PR  ftime(&tfinish);
//PR  printf("Startup time was %2.2f seconds. \n"
//PR    ,difftime(tfinish.time,tstart.time));

}

/*******************************************************************/
readparfile()
/*******************************************************************/
{
char line[512];
char par[512];
char value[512];
char term[2]=" ";
int result;
char *pdest;
int cc,cc1;
char temp[6];

int baud;
int databits;
char parity;
int stopbits;

  strcpy(full_param_file,PARAMETER_FILE);
  printf("\nUsing parameter file: %s\n",full_param_file);
  if((fpp=fopen(full_param_file,"rt"))==NULL){
     printf("Parameter file was not found, will use defaults\n");
     posfm++;posfd++;
     /* write to message file */
     /* this is normal, so don't increment the posfm counter */
     fp=fopen(mesposlog,"a");
     fprintf(fp,"%i,%s parameter file %s was not found, will use defaults\n",posfd,scurdatetim,PARAMETER_FILE);
     fclose(fp);
     return(0);}
  /* read lines until we hit eof */
  
  for(;;){
    if(fgets(line,511,fpp)==NULL){
      fclose(fpp); 
      return(0);}  
    printf("%s",line);  /* print each line */
    /* write to message files */
    /* this is normal, so don't increment the posfm counter */
    fp=fopen(mesposlog,"a");
    fprintf(fp,"%i,%s %s",posfd,scurdatetim,line);
    fclose(fp);
    
    /* break out the parameter and its value */
    /* process if it is not a comment */
    /* comments must start with a ';' */
    /* 27 Feb 03 allow white space in comments */
    if(strncmp(line, ";", 1)!=0){
     pdest=strchr(line,'=');
     cc=pdest-line+1;
     strncpy(par,line,cc);  /* get the parameter string */
     strcpy(par+cc,term+1);       /* add a null terminator to par string*/
     sscanf(line+cc,"%s",value);  /* get the value string */
     //printf("<%s> <%s>\n",par,value);
    
    /* compare the par string with possible pars */
    if(strcmp(par,"POSACQ1.PAR_updated=")==0){
      sscanf(value,"%s",pardate);
      }
    else if(strcmp(par,"serial_port=")==0){
      //note: scanf awaits a whitespace terminated string when it tries to read %s.
      //      specify the non-matching character directly via [^ ]
      sscanf(value,"%[^,],%d,%d,%c,%d",SERIAL_PORT,&baud,&databits,&parity,&stopbits);
      //printf("%s ; %d ; %d ; %c ; %d\n",SERIAL_PORT,baud,databits,parity,stopbits);
      switch (baud) {
      case  1200: SERIAL_BAUDRATE=B1200; break;
      case  2400: SERIAL_BAUDRATE=B2400; break;
      case  4800: SERIAL_BAUDRATE=B4800; break;
      case  9600: SERIAL_BAUDRATE=B9600; break;
      case 19200: SERIAL_BAUDRATE=B19200; break;
      case 38400: SERIAL_BAUDRATE=B38400; break;
      case 57600: SERIAL_BAUDRATE=B57600; break;
      default: fprintf(stderr,"Unsupported baud rate\n"); return(0);
      }
      switch (databits) {
      case 7: SERIAL_BIT = CS7; break;
      case 8: SERIAL_BIT = CS8; break;
      default: fprintf(stderr,"Unsupported data size\n"); return(0);
      }
      switch (parity) {
      case 'n':
      case 'N': SERIAL_PARITY = ~PARENB; break; // Clear parity enable
      case 'o':
      case 'O': SERIAL_PARITY = PARENB | PARODD; break; // Enable, odd parity 
      case 'e':
      case 'E': SERIAL_PARITY = PARENB & ~PARODD; break; // Enable, turn off odd parity = even
      default: fprintf(stderr,"Unsupported parity\n"); return(0);
      }
      if(stopbits=1) SERIAL_STOP=~CSTOPB; else SERIAL_STOP=CSTOPB;
      //printf("%s ; %d ; %d ; %d ; %d\n",SERIAL_PORT,SERIAL_BAUDRATE,SERIAL_BIT,SERIAL_PARITY,SERIAL_STOP);
      }
    else if(strcmp(par,"site=")==0){
      sscanf(value,"%s",site);
      }
    else if(strcmp(par,"maildest=")==0){
      sscanf(value,"%s",maildest);
         /* replace ';' with ' ' in multiple email destinations */
         result=strlen(maildest);
         for(i=0; i<result; i++){
            if(maildest[i]==';') maildest[i]=' ';
         }
      }
    else if(strcmp(par,"possendmail=")==0){
      sscanf(value,"%c",&possendmail);
      }
    else {
      printf("unknown parameter\n");
      /* write to message files */
      /* this is not normal, so increment the posfm counter */
      posfm++;posfd++;
      fp=fopen(mesposlog,"a");
      fprintf(fp,"%i,%s unknown parameter\n",posfd,scurdatetim);
      fclose(fp);
      }

     } /* end of if not a comment */
      
  } /* end of the read for loop */
} /* end of readparfile() */

/*******************************************************************/
init_serial_port()
/*******************************************************************/
{
  /* initialize and open SERIAL_PORT for POSS packets */ 
  /* 9600 baud, 7 bit, even parity, 1 stop */
  /* try PARENB and then ignore it with IGNPAR */
  fhR2=open(SERIAL_PORT, O_RDWR | O_NOCTTY | O_NDELAY); 
  if(fhR2<0) {perror(SERIAL_PORT); exit(-1);}
  
  //http://digilander.libero.it/robang/rubrica/serial.htm
  //Timeouts are ignored in canonical input mode or when the NDELAY option is set on the file via open or fcntl.
  //fcntl(fhR2, F_SETFL, FNDELAY);

  bzero(&newtio, sizeof(newtio));
  //tcgetattr(fhR2, &newtio);
  cfsetispeed(&newtio, SERIAL_BAUDRATE);
  cfsetospeed(&newtio, SERIAL_BAUDRATE);
  
// [radar@POSS2 POSACQ1]$ stty -a -F /dev/ttyS0
// speed 9600 baud; rows 0; columns 0; line = 0;
// intr = <undef>; quit = <undef>; erase = <undef>; kill = <undef>; eof = <undef>; eol = <undef>; eol2 = <undef>; swtch = <undef>; start = <undef>; stop = <undef>; susp = <undef>; rprnt = <undef>; werase = <undef>; lnext = <undef>;
// flush = <undef>; min = 1; time = 5;
//c_cflag: parenb -parodd cs7 hupcl -cstopb cread clocal -crtscts -cdtrdsr
//c_iflag: ignbrk -brkint -ignpar -parmrk -inpck -istrip -inlcr -igncr -icrnl -ixon -ixoff -iuclc -ixany -imaxbel -iutf8
//c_oflag: -opost -olcuc -ocrnl -onlcr -onocr -onlret -ofill -ofdel nl0 cr0 tab0 bs0 vt0 ff0
//c_lflag: -isig -icanon -iexten -echo -echoe -echok -echonl -noflsh -xcase -tostop -echoprt -echoctl -echoke
  
  newtio.c_cflag = SERIAL_BIT | SERIAL_PARITY | CLOCAL | CREAD;
  newtio.c_iflag = IGNBRK; //ignore breaks
  newtio.c_oflag = 0;
  newtio.c_lflag = 0;      // non-canonical, no echo ..

  tcflush(fhR2, TCIFLUSH);
  if(tcsetattr(fhR2, TCSANOW, &newtio) != 0) {perror("tcsetattr failed"); exit(-1);}
  printf("Serial port, %s, initialization successful.\n",SERIAL_PORT);

}

/*******************************************************************/
checkposs()
/*******************************************************************/
{
char char2;
  /* data is 9600, 7,E,1, 1 minute, variable length, between SOH and EOT 
   packets start with 0D 0A 01  (CR LF SOH)
   packets end with   0A 0D 04  (LF CR EOT)

   then we get an optional histogram
    packets start with 02 (STX)
    packets end with 0D 0A 04 0A 0D 00 (CR LF EOT LF CR NULL) */

  /* the poss output messages cannot by syncd to the PC clock. */
  
     
  /* read 1 char at a time until we have a packet 
           (ends in EOT 0x04), or no more char */
  do {
        nchar2=read(fhR2,&char2,1);
     if(nchar2<0) perror("read failed");
           if(nchar2==1) {
       if(posrawd==1){
         if     (char2==0x01) printf("posrawd pchar2=%i char2 %2.2X <%c> SOH\n",pchar2,char2,char2);
         else if(char2==0x02) printf("posrawd pchar2=%i char2 %2.2X <%c> STX\n",pchar2,char2,char2);
         else if(char2==0x04) printf("posrawd pchar2=%i char2 %2.2X <%c> EOT\n",pchar2,char2,char2);
         else                 printf("posrawd pchar2=%i char2 %2.2X <%c>\n",pchar2,char2,char2);
       }
       //sposs[pchar2]=char2;  /* keep all chars */ 
       /*don't start saving chars until we have a 0x01(SOH) or 0x02(STX) */
       if(pchar2>0){sposs[pchar2]=char2; pchar2++;}
       if(pchar2==0 && (char2==0x01||char2==0x02)){sposs[pchar2]=char2; pchar2++;}
  
     } /* end of if nchar2==1 */
  }
  while (nchar2>0 && char2!=0x04 && pchar2<8192); /* 0x04 is the EOT that terminates both packets*/
  /* we do not store the 0D 0A before the SOH of the PS091 or the trailing LF CR NULL of the histogram */

  if(nchar2<=0) return(1);   /* we ran out of data */

  /* we have an EOT, or pchar2=8192 without an EOT;  process the line */

//  for(i=1; i<=pchar2; i++) printf("%c ",sposs[i]);
//  printf("\n");

  sposs[pchar2]=0;  /* put a null at the end of the line */
  /* if packet doesn't start with 01(SOH) or 02(STX) and end with 04(EOT),  throw out the line */
  if( (sposs[0]!=0x01&&sposs[0]!=0x02) || sposs[pchar2-1]!=0x04) { /* bad packet */
     posfm++;posfd++;
     printf("%i,poss packet not between 0x01/2 and 0x04, %2.2X %2.2X\n",posfd,sposs[0],sposs[pchar2-1]);
     /* write to message file */
     fp=fopen(mesposlog,"a");
     fprintf(fp,"%i,%s poss packet not between 0x01/2 and 0x04, %2.2X %2.2X\n",posfd,scurdatetim,sposs[0],sposs[pchar2-1]);
     fclose(fp);
     pchar2=0; 
     return(0);
  }

  /* *************** we have a good packet (either PS091 or histogram) *****************/
  minnopos=0;

  /* if this a PS091 packet, insert the current computer time */  
  /* format SPS091 yymmdd hh:mm */
  if(sposs[0]==0x01){
    sposs[7]=scurdatetim[8];  /* tens years */
    sposs[8]=scurdatetim[9];  /* units years */
    sposs[9]=scurdatetim[3];  /* tens months */
    sposs[10]=scurdatetim[4];  /* units months */
    sposs[11]=scurdatetim[0];  /* tens days */
    sposs[12]=scurdatetim[1];  /* units days */
    sposs[13]=' ';      /*    */
    sposs[14]=scurdatetim[11];  /* tens hours */
    sposs[15]=scurdatetim[12];  /* units hours */
    sposs[16]=':';      /* :  */
    sposs[17]=scurdatetim[14];  /* tens mins */
    sposs[18]=scurdatetim[15];  /* units mins */
  } /* end of if 0x01 */

  /* if this is a PS091 packet, check for non-zero fault code */
  if(sposs[0]==0x01){
  /* watch out, the line might be variable length, fault code may be beyond posn 86 sometimes */
  strncpy(posfaultcode,sposs+86,4);
  posfaultnum=0x0000;   /* added this line 01 Aug 07 */
  if(strncmp(posfaultcode, "0000",4) != 0){
     sscanf(posfaultcode,"%X",&posfaultnum);
     /* expect lots of transient noise faults, and some others,
        only send email message (posfm>0) on a few of the critical faults */
     posfd++;
     for(i=0;i<=511;i++) posfault[i]=0; /* clear the string*/
     if((posfaultnum & 0x8000) == 0x8000) strcat(posfault,",min noise limit");
     if((posfaultnum & 0x4000) == 0x4000) strcat(posfault,",max noise limit");
     if((posfaultnum & 0x2000) == 0x2000) strcat(posfault,",transient noise");
     if((posfaultnum & 0x1000) == 0x1000) {strcat(posfault,",constant noise");posfm++;}
     if((posfaultnum & 0x0800) == 0x0800) strcat(posfault,",false alarm");
     if((posfaultnum & 0x0400) == 0x0400) strcat(posfault,",Gunn current low");
     if((posfaultnum & 0x0200) == 0x0200) strcat(posfault,",not used");
     if((posfaultnum & 0x0100) == 0x0100) strcat(posfault,",air temp ");
     if((posfaultnum & 0x0080) == 0x0080) strcat(posfault,",heater voltage");
     if((posfaultnum & 0x0040) == 0x0040) strcat(posfault,",Gunn voltage");
     if((posfaultnum & 0x0020) == 0x0020) {strcat(posfault,",-12V");posfm++;}
     if((posfaultnum & 0x0010) == 0x0010) {strcat(posfault,",+12V");posfm++;}
     if((posfaultnum & 0x0008) == 0x0008) {strcat(posfault,",+5V");posfm++;}
     if((posfaultnum & 0x0004) == 0x0004) strcat(posfault,",heater current");
     if((posfaultnum & 0x0002) == 0x0002) strcat(posfault,",not used");
     if((posfaultnum & 0x0001) == 0x0001) strcat(posfault,",DC offset");
     printf("\n%i,%s poss fault code=%s %s\n",posfd,scurdatetim,posfaultcode,posfault);
     /* write to message file */
     fp=fopen(mesposlog,"a");
     fprintf(fp,"%i,%s poss fault code=%s %s\n",posfd,scurdatetim,posfaultcode,posfault);
     fclose(fp);
  }  /* end of if posfaultcode != 0 */
  } /* end of if 0x01 */

  /* print packet if posd=1 (a kybd command) */
  if(posd==1){
//   printf("POS nCHARs=%i\n%s\n",pchar2,sposs);
   printf("%s",sposs);
  }

  /* write packet to file */
  fp=fopen(poslog,"a");
  fprintf(fp,"%s",sposs);
  fclose(fp);

  pchar2=0;  /* reset the char counter */

} /* end of checkposs() */

/*******************************************************************/
checkkybd()
/*******************************************************************/
{
int i,nh;
char tempstr[81];
int nread;
char ch;

/* read keyboard until we have no more keyboard hits */
  do{
  nread=read(0,&ch,1);
  if(nread==1){  /* we've read a character */
     if(kbptr>80) kbptr=0;  /* expect cr within 80 chars */
     if(ch==0x03) exit(0);  /* quit on ctrl-c */
     kbchar=tolower(ch);
     kbword[kbptr++]=kbchar;
     if(kbchar==0x0A) {    /* process a word */
       kbword[kbptr]=0;    /* append a NULL*/
  printf("\ngot word %s\n",kbword);  

  if(memcmp(kbword,"help",4)==0){ printf("\n");
  printf("help    - list keyboard commands\n");
  printf("posdon    - print all poss status lines\n");
  printf("posrawdon - print all poss raw characters\n");
  printf("comment   - add an operator comment to %s\n",mespos_basefile);
  printf("quit    - exit program\n");
  }

  else if(memcmp(kbword,"posdon", 6)==0) posd=1;
  else if(memcmp(kbword,"posdoff",7)==0) posd=0;
  else if(memcmp(kbword,"posrawdon",  9)==0) posrawd=1;
  else if(memcmp(kbword,"posrawdoff",10)==0) posrawd=0;
  else if(memcmp(kbword,"comment",7)==0){
   memccpy(tempstr,&kbword[7],'\n',72);  /* get comment, on same line, max 72 chars */
   fp=fopen(mesposlog,"a");
   fprintf(fp,"%i,%s comment: <%s>\n",posfd,scurdatetim,tempstr);
   fclose(fp);
  }
  else if(memcmp(kbword,"quit",4)==0){
   posfm++;posfd++;
   /* write to message file */
   fp=fopen(mesposlog,"a");
   fprintf(fp,"%i,%s keyboard quit command\n",posfd,scurdatetim);
   fclose(fp);
   exit(0);
  }
  else  printf("Not a valid command word.\n");

  /* clear out kbword */
  for(i=0;i<80;i++) kbword[i]=0;
  kbptr=0;
  } /* end of process a word  */

  } /* end of read a character */
  
  } /* end of do */
  while(nread>0);  
  
} /* end of checkkybd() */

/*******************************************************************/
mkposnm()
/*******************************************************************/
{
  strcpy(pos_basefile,POSS_DAT_FILE);

/*  make today's poss file name: POSyyyymmdd.csv*/
  ftime(&longtime);
  newtime=localtime(&longtime.time);
  acqtimline=ctime(&longtime.time);
  if(acqtimline[8]==' ') acqtimline[8]='0';  /* gcc ctime uses space */
  strncpy(pos_basefile+3,acqtimline+20,4);  /* 4 digit year */
  month=newtime->tm_mon;    /* month 0-11 */
  month+=1;      /* month 1-12 */
  pos_basefile[7]=(month/10)+0x30;  /* tens of month */
  pos_basefile[8]=(month%10)+0x30;  /* units of month */
  pos_basefile[9]=acqtimline[8];  /* tens  day */
  pos_basefile[10]=acqtimline[9];  /* units day */
  olddayt=acqtimline[8];    /* save to test for day change */
  olddayu=acqtimline[9];    /* save to test for day change */
  strcpy(poslog,DATA_PATH);
  strcat(poslog,pos_basefile);
  printf("Today's poss file name is: %s\n",poslog);
  /* open the file to create it */
  fp=fopen(poslog,"a");
  fclose(fp);
//   /*in case we are root, change the ownership to user_account */
//   strcpy(syscmd,"chown %s ",user_account); 
//   strcat(syscmd,poslog);
//   printf("<%s>\n",syscmd);
//   system(syscmd);  
} /* end of mkposnm */

/*******************************************************************/
mkmesnm()
/*******************************************************************/
{
  strcpy(mespos_basefile,POSS_MSG_FILE);

  //  make today's poss message file name: MESPOSyyyymmdd.txt
  ftime(&longtime);
  newtime=localtime(&longtime.time);
  acqtimline=ctime(&longtime.time);
  if(acqtimline[8]==' ') acqtimline[8]='0';  /* gcc ctime uses space */
  strncpy(mespos_basefile+6,acqtimline+20,4);  /* 4 digit year */
  month=newtime->tm_mon;    /* month 0-11 */
  month+=1;      /* month 1-12 */
  mespos_basefile[10]=(month/10)+0x30;  /* tens of month */
  mespos_basefile[11]=(month%10)+0x30;  /* units of month */
  mespos_basefile[12]=acqtimline[8];  /* tens  day */
  mespos_basefile[13]=acqtimline[9];  /* units day */
  olddayt=acqtimline[8];    /* save to test for day change */
  olddayu=acqtimline[9];    /* save to test for day change */
  strcpy(mesposlog,DATA_PATH);
  strcat(mesposlog,mespos_basefile);
  printf("Today's message file name is: %s\n",mesposlog);
  /* open the file to create it */
  fp=fopen(mesposlog,"a");
  fclose(fp);
//   /*in case we are root, change the ownership to user_account */
//   strcpy(syscmd,"chown %s ",user_account); 
//   strcat(syscmd,mesposlog);
//   printf("<%s>\n",syscmd);
//   system(syscmd);  

/*
//  make today's vis  message  file name  MESVISyyyymmdd.txt
  ftime(&longtime);
  newtime=localtime(&longtime.time);
  acqtimline=ctime(&longtime.time);
  if(acqtimline[8]==' ') acqtimline[8]='0'; 
  strncpy(mesvisfile+6,acqtimline+20,4);  
  month=newtime->tm_mon;    
  month+=1;      
  mesvisfile[10]=(month/10)+0x30;  
  mesvisfile[11]=(month%10)+0x30;  
  mesvisfile[12]=acqtimline[8];  
  mesvisfile[13]=acqtimline[9];  
  olddayt=acqtimline[8];    
  olddayu=acqtimline[9];    
  strcpy(mesvislog,mesvispath);
  strcat(mesvislog,mesvisfile);
  printf("Today's message   file name is: %s\n",mesvislog);
  // open the file to create it 
  fp=fopen(mesvislog,"a");
  fclose(fp);
  //in case we are root, change the ownership to operator 
  strcpy(syscmd,"chown radar "); //PR: ### Korean operator ###
  strcat(syscmd,mesvislog);
  printf("<%s>\n",syscmd);
  system(syscmd);  
*/
} /* end of mkmesnm */

/*******************************************************************/
testday()
/*******************************************************************/
{
  /* read the current time */
  ftime(&longtime);
  newtime=localtime(&longtime.time);
   acqtimline=ctime(&longtime.time);
  if(acqtimline[8]==' ') acqtimline[8]='0';  /* gcc ctime uses space */
  if(acqtimline[8]!=olddayt || acqtimline[9]!=olddayu){
   /* the units of day has changed */

   /* make new log file name */
   mkposnm();

   /* clear the fail counter */
   posfd=0;

  } /* end of day has changed */

} /* end of testday() */

/*******************************************************************/
testhour()
/*******************************************************************/
{
  /* read the current time */
  ftime(&longtime);
  newtime=localtime(&longtime.time);
   acqtimline=ctime(&longtime.time);
  if(acqtimline[8]==' ') acqtimline[8]='0';  /* gcc ctime uses space */
  if(acqtimline[12]!=oldhr){
   /* the units of hours has changed */
   posmailsent=0;
   oldhr=acqtimline[12];
  }

} /* end of testhours() */

/*******************************************************************/
testmin()
/*******************************************************************/
{
  /* read the current time */
  ftime(&longtime);
  newtime=localtime(&longtime.time);
   acqtimline=ctime(&longtime.time);
  if(acqtimline[8]==' ') acqtimline[8]='0';  /* gcc ctime uses space */
  if(acqtimline[15]!=oldmin){
   /* the units of mins has changed */
   minnopos++;  /* increment minutes with no valid data */

   if(minnopos >= 3){
     posfm++;posfd++;
     /* write to message file */
     printf("%i,%s no good poss packets in %i minutes.\n",posfd,scurdatetim,minnopos);
     fp=fopen(mesposlog,"a");
     fprintf(fp,"%i,%s no good poss packets in %i minutes.\n",posfd,scurdatetim,minnopos);
     fclose(fp);
     } /* end of minnopos>=3 */

    /* check for poss fails this minute */
    if(posfm >=1){
      printf("posfm=%i posfd=%i\n",posfm,posfd);
      // send email message
      posemail();
    } // end of posfm>=1

   //printf("nloops=%i\n",nloops);
   nloops=0;

   oldmin=acqtimline[15];
   posfm=0;
  } /* end of mins has changed */

} /* end of testmin() */

/*******************************************************************/
curdatetim()
/*******************************************************************/
{
  /* make date time string in format: char scurdatetim[21]="dd/mm/yyyy,hh:mm:ss," */
  ftime(&tstart);  /* get system time */
  acqtimline=ctime(&tstart.time);
  scurdatetim[0]=acqtimline[8];  /* tens days */
  scurdatetim[1]=acqtimline[9];  /* units days */
  scurdatetim[2]='/';  
  newtime=localtime(&tstart.time);/* month is ugly to get */
  month=newtime->tm_mon;    /* month 0-11 */
  month+=1;      /* month 1-12 */
  scurdatetim[3]=(month/10)+0x30;  /* tens months */
  scurdatetim[4]=(month%10)+0x30;  /* units months */
  scurdatetim[5]='/';
  scurdatetim[6]=acqtimline[20];  /* thous years */
  scurdatetim[7]=acqtimline[21];  /* hunds years */
  scurdatetim[8]=acqtimline[22];  /* tens years */
  scurdatetim[9]=acqtimline[23];  /* units years */
  scurdatetim[10]=',';  
  scurdatetim[11]=acqtimline[11];  /* tens hours */
  scurdatetim[12]=acqtimline[12];  /* units hours */
  scurdatetim[13]=':';  
  scurdatetim[14]=acqtimline[14];  /* tens minutes */
  scurdatetim[15]=acqtimline[15];  /* units minutes */
  scurdatetim[16]=':';  
  scurdatetim[17]=acqtimline[17];  /* tens secs */
  scurdatetim[18]=acqtimline[18];  /* units secs */
  scurdatetim[19]=',';  
}  /* end of curdatetime() */

/*******************************************************************/
posemail()
/*******************************************************************/
{
char mailcmd[570];

  /* only send one par email message per hour */
  if(posmailsent==1) return(0);

  /* only send email if requested */
  if(possendmail!='Y') return(0);

  /* make the mail message file */
  strcpy(full_mail_file,MAIL_MSG_FILE);
  fp=fopen(full_mail_file,"w");
  fprintf(fp,"posfm=%i posfd=%i\n",posfm,posfd);
  fclose(fp);
  strcpy(mailcmd, "tail ");
  strcat(mailcmd, mesposlog);
  strcat(mailcmd, " >> ");
  strcat(mailcmd, full_mail_file);
  system(mailcmd);
  printf("email message <%s>\n",mailcmd);
  /* make email command */
  strcpy(mailcmd,"mail");
  strcat(mailcmd," -sposfail");
  strcat(mailcmd," ");
  strcat(mailcmd, maildest);
  strcat(mailcmd," < ");
  strcat(mailcmd, full_mail_file);
  system(mailcmd);
  printf("sent posfail email message <%s>\n",mailcmd);

  posmailsent=1;  /* sent a pos email message this hour */

} /* end of posemail() */
/**********************************************************/

/* the end of program */
